/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Hydrogen.h,v 1.46 2003/09/26 09:24:22 comix Exp $
 *
 */
#ifndef HYDROGEN_H
#define HYDROGEN_H

#include "Song.h"
#include "Object.h"
#include "GenericDriver.h"
#include "EngineListener.h"

// Engine state
#define UNINITIALIZED	1
#define INITIALIZED		2
#define PREPARED		3
#define READY			4
#define PLAYING		5


// forward
class AlsaMidiDriver;

//----------------------------------------------------------------------------
/**
 * Hydrogen Audio Engine
 */
class Hydrogen : public Object
{
	public:
		virtual string getClassName() {	return "Hydrogen";	}

		/** Return the Hydrogen instance */
		static Hydrogen* getInstance(EngineListener *listener = NULL);

		/** Destructor */
		~Hydrogen();

		/** Start the internal sequencer */
		int start();

		/** Stop the internal sequencer */
		void stop();

		/** Set current song */
		void setSong(Song *newSong);

		/** Return the current song */
		Song* getSong();
		void removeSong();

		void noteOn(Note *note);
		void noteOff(Note *note);

		float getMasterPeak_L();
		void setMasterPeak_L(float value);

		float getMasterPeak_R();
		void setMasterPeak_R(float value);

		float getDelayBufferPeak_L();
		void setDelayBufferPeak_L(float value);

		float getDelayBufferPeak_R();
		void setDelayBufferPeak_R(float value);

		unsigned long getTickPosition();

		void setNextPattern(int pos);

		Pattern* getCurrentPattern();
		void setCurrentPattern(Pattern *newPattern);

		uint getPatternPos();
		void setPatternPos( uint pos );

		uint getPlayingNotes();

		void restartDrivers();

		uint exportSong(string filename);

		uint getSongNotesQueue();

		GenericDriver* getAudioDriver();
		AlsaMidiDriver* getMidiDriver();

		int getState();

		void addEngineListener(EngineListener *newListener);

		void lockEngine();
		void unlockEngine();

		float getProcessTime();
		float getMaxProcessTime();

		int loadDrumkit( DrumkitInfo *drumkitInfo );

		void raiseError(uint nErrorCode);

		void raiseMidiActivityEvent();

		unsigned long getTotalFrames();

		enum {
			NO_OSS_SUPPORT,
			NO_JACK_SUPPORT,
			UNKNOWN_DRIVER,
			ERROR_STARTING_DRIVER,
			JACK_SERVER_SHUTDOWN,
			JACK_CANNOT_ACTIVATE_CLIENT,
			JACK_CANNOT_CONNECT_OUTPUT_PORT,
			JACK_ERROR_IN_PORT_REGISTER
		};
	private:
		static Hydrogen* instance;

		/** Constructor */
		Hydrogen( EngineListener *mainListener );

};

#endif

