/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: PreferencesMng.h,v 1.27 2003/11/07 18:13:26 comix Exp $
 *
 */

#ifndef PREFERENCES_MNG
#define PREFERENCES_MNG

#include <string>
using std::string;

#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <fstream>
#include <iostream>
#include <dirent.h>
#include <stdio.h>

#include "Object.h"

const float FALLOFF_SLOW = 	1.08;
const float FALLOFF_NORMAL=	1.1;
const float FALLOFF_FAST =	1.5;

// forward declaration
class TiXmlNode;

class WindowProperties : public Object{
	public:
		virtual string getClassName() {	return "WindowProperties";	}
		int x;
		int y;
		int width;
		int height;

		WindowProperties();
		~WindowProperties();
};



//----------------------------------------------------------------------------
/**
 * Manager for User Preferences File (singleton)
 */
class PreferencesMng : public Object {
	public:
		/** Return an instance of PreferencesMng */
		static PreferencesMng* getInstance();

		/** Destructor */
		~PreferencesMng();

		virtual string getClassName() {	return "PreferencesMng";	}

		/** Load the preferences file */
		void loadPreferences();

		/** Save the preferences file */
		void savePreferences();

		void setMaxNotes(uint notes) {	maxNotes = notes;	}
		uint getMaxNotes() {	return maxNotes;	}

		/** Return the audio buffer size */
		uint getBufferSize() {	return bufferSize;	}

		/** Set the audio buffer size */
		void setBufferSize(uint bufferSize) {	this->bufferSize = bufferSize;	}

		/** return the selected audio driver */
		string getAudioDriver() {	return audioDriver;	}

		/** Selected an audio driver */
		void setAudioDriver(string audioDriver) {	this->audioDriver = audioDriver;	}

		/** return sample rate */
		uint getSampleRate() {	return sampleRate;	}

		/** set sample rate */
		void setSampleRate(uint sampleRate) {	this->sampleRate = sampleRate;	}

		/** set OSS device */
		void setOssDevice(string device) {	ossDevice = device;	}

		/** return the OSS device */
		string getOssDevice() {	return ossDevice;	}


		/** return if metronome is enabled */
		bool isMetronomeEnabled() {	return useMetronome;	}

		/** Set if the metronome must be active */
		void setUseMetronome(bool useMetronome) {	this->useMetronome = useMetronome;	}

		/** return the metronome volume */
		float getMetronomeVolume() {	return metronomeVolume;	}

		/** set the metronome volume */
		void setMetronomeVolume(float metronomeVolume) {	this->metronomeVolume = metronomeVolume;	}

		string getApplicationFontFamily() {	return applicationFontFamily;	}
		void setApplicationFontFamily(string family) {	applicationFontFamily = family;	}

		int getApplicationFontPointSize() {	return applicationFontPointSize;	}
		void setApplicationFontPointSize(int size) {	applicationFontPointSize = size;	}

		string getMixerFontFamily() {	return mixerFontFamily;	}
		void setMixerFontFamily(string family) {	mixerFontFamily = family;	}

		int getMixerFontPointSize() {	return mixerFontPointSize;	}
		void setMixerFontPointSize(int size) {	mixerFontPointSize = size;	}

		float getMixerFalloffSpeed() {	return mixerFalloffSpeed;	}
		void setMixerFalloffSpeed(float value) {	mixerFalloffSpeed = value;	}

		string getDemoPath() {	return demoPath;	}

		string getInterfaceMode() {	return interfaceMode;	}
		void setInterfaceMode( string mode ) {	interfaceMode = mode;	}

		string getJackPortName1() {	return jackPortName1;	}
		string getJackPortName2() {	return jackPortName2;	}

		void setJackPortName1( string port ) {	jackPortName1 = port;	}
		void setJackPortName2( string port ) {	jackPortName2 = port;	}

		bool isJackTransportSlave() {	return jackTransportSlave; }
		void setJackTransportSlave( bool slave ) {	jackTransportSlave = slave;	}

		bool isJackConnectDefaults() {	return jackConnectDefaults; }
		void setJackConnectDefaults( bool slave ) {	jackConnectDefaults = slave;	}

		bool isJackTrackOuts() {	return jackTrackOuts; }
		void setJackTrackOuts( bool slave ) {	jackTrackOuts = slave;	}

		int getMidiPortChannel() {	return midiPortChannel;	}
		void setMidiPortChannel(int channel) {	midiPortChannel = channel;	}

		string getMidiDest_name() {	return midiDest_name;	}
		int getMidiDest_client() {	return midiDest_client;	}
		int getMidiDest_port() {	return midiDest_port;	}

		void setMidiDest_name( string name ) {	this->midiDest_name = name;	}
		void setMidiDest_client( int client ) {	this->midiDest_client = client;	}
		void setMidiDest_port( int port ) {	this->midiDest_port = port; }


		/** Get user's home directory. */
		string getUserHome();

		WindowProperties readWindowProperties( TiXmlNode *parent, string windowName, WindowProperties defaultProp );
		void writeWindowProperties( TiXmlNode *parent, string windowName, WindowProperties prop );

		WindowProperties getMainFormProperties() {	return mainFormProperties;	}
		void setMainFormProperties( WindowProperties prop ) {	mainFormProperties = prop;	}

		WindowProperties getMixerProperties() {	return mixerProperties;	}
		void setMixerProperties( WindowProperties prop ) {	mixerProperties = prop;	}

		WindowProperties getPatternEditorProperties() {	return patternEditorProperties;	}
		void setPatternEditorProperties( WindowProperties prop ) {	patternEditorProperties = prop;	}

		WindowProperties getSongEditorProperties() {	return songEditorProperties;	}
		void setSongEditorProperties( WindowProperties prop ) {	songEditorProperties = prop;	}

		WindowProperties getDrumkitManagerProperties() {	return drumkitManagerProperties;	}
		void setDrumkitManagerProperties( WindowProperties prop ) {	drumkitManagerProperties = prop;	}

		WindowProperties getAudioEngineInfoProperties() {	return audioEngineInfoProperties;	}
		void setAudioEngineInfoProperties( WindowProperties prop ) {	audioEngineInfoProperties = prop;	}

		void setLastSongFilename( string filename ) {	lastSongFilename = filename;	}
		string getLastSongFilename() {	return lastSongFilename;	}

		void setRestoreLastSongEnabled( bool restore ) {	restoreLastSong = restore;	}
		bool isRestoreLastSongEnabled() {	return restoreLastSong;	}

		void setHearNewNotes( bool value ) {	hearNewNotes = value;	}
		bool getHearNewNotes() {	return hearNewNotes;	}

		int getPatternEditorGridResolution() {	return m_nPatternEditorGridResolution;	}
		void setPatternEditorGridResolution( int value ) {	m_nPatternEditorGridResolution = value;	}

		bool isPatternEditorUsingTriplets() {	return m_bPatternEditorUsingTriplets;	}
		void setPatternEditorUsingTriplets( bool value ) {	m_bPatternEditorUsingTriplets = value;	}

	private:
		static PreferencesMng *instance;

		/** directory of demo songs */
		string demoPath;

		//___ General properties ___

		/** restore last song? **/
		bool restoreLastSong;

		/** last song used **/
		string lastSongFilename;

		bool hearNewNotes;

		/// max notes
		uint maxNotes;


		//___ audio engine properties ___
		/** Audio driver */
		string audioDriver;

		/** Use metronome? */
		bool useMetronome;

		/** Metronome volume */
		float metronomeVolume;



		//___ oss driver properties ___
		/** Audio buffer size */
		uint bufferSize;

		/** Audio sample rate */
		uint sampleRate;

		/** device used for output */
		string ossDevice;


		//___  jack driver properties ___
		string jackPortName1;
		string jackPortName2;
		bool jackTransportSlave;
		bool jackTrackOuts;
		bool jackConnectDefaults;

		//___ alsa midi driver properties ___
		int midiPortChannel;
		string midiDest_name;
		int midiDest_client;
		int midiDest_port;



		//___ GUI properties ___
		/** Interface mode */
		string interfaceMode;

		string applicationFontFamily;
		int applicationFontPointSize;

		string mixerFontFamily;
		int mixerFontPointSize;

		float mixerFalloffSpeed;

		int m_nPatternEditorGridResolution;
		bool m_bPatternEditorUsingTriplets;

		WindowProperties mainFormProperties;
		WindowProperties mixerProperties;
		WindowProperties patternEditorProperties;
		WindowProperties songEditorProperties;
		WindowProperties drumkitManagerProperties;
		WindowProperties audioEngineInfoProperties;



		/** Constructor */
		PreferencesMng();

		/** Create preferences directory */
		void createPreferencesDirectory();

		/** Create data directory */
		void createDataDirectory();

		void writeXmlText( TiXmlNode *parent, string name, string text );
		void writeXmlText( TiXmlNode *parent, string name, float value );
		void writeXmlText( TiXmlNode *parent, string name, int value );

		string readXmlString( TiXmlNode* parent, string nodeName, string defaultValue );
		bool readXmlBool( TiXmlNode* parent, string nodeName, bool defaultValue );
		float readXmlFloat( TiXmlNode* parent, string nodeName, float defaultValue );
		int readXmlInt( TiXmlNode* parent, string nodeName, int defaultValue );

};

#endif

